/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package abstrata;

import java.awt.Image;
import java.awt.Rectangle;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 *
 * @author Emerson Ribeiro de Mello
 * @author Fernando Fernandes Simão
 */
public abstract class Personagem {

    protected Image imagem; // onde será carregada a imagem do jogador (usado pela Tela)
    public JPanel tela; // para interagir com o JPanel onde está inserido
    public int x; // coordenada X atual do jogador
    public int y;  // coordenada Y atual do jogador
    protected int largura; // largura da imagem que fora carregada
    protected int altura; // altura da imagem que fora carregada
    protected int dirX; // direção no eixo X que o objeto deverá se movimentar
    protected int dirY; // direção no eixo Y que o objeto deverá se movimentar
    protected int larguraTela;
    protected int alturaTela;

    public Personagem(String nomeImagem, JPanel jp, int px, int py) {
        ImageIcon iicon = new ImageIcon(this.getClass().getResource(nomeImagem));
        this.imagem = iicon.getImage();
        this.tela = jp;
        this.x = px;
        this.y = py;
        this.largura = imagem.getWidth(null);
        this.altura = imagem.getHeight(null);
        
    }

    /**
     * Altera a imagem atual
     * @param nomeImagem endereço da imagem
     */
    public void alterarImagem(String nomeImagem) {
        ImageIcon iicon = new ImageIcon(this.getClass().getResource(nomeImagem));
        this.imagem = iicon.getImage();
        this.largura = imagem.getWidth(null);
        this.altura = imagem.getHeight(null);

    }

    /**
     *  Movimentação do personagem
     */
    public void mover() {
        // obtendo a largura e altura da tela
        larguraTela = this.tela.getSize().width;
        alturaTela = this.tela.getSize().height;

        //teste para evitar que ultrapasse os limites da tela - eixo X
        x += (((dirX > 0) && (x + dirX < larguraTela - this.largura)) || ((dirX < 0) && (x - dirX > 0))) ? dirX : 0;
        if (x < 0) {
            x = 0;
        }
        //teste para evitar que ultrapasse os limites da tela - eixo Y
        y += (((dirY > 0) && (y + dirY < alturaTela - this.altura)) || ((dirY < 0) && (y - dirY > 0))) ? dirY : 0;
        if (y < 0) {
            y = 0;
        }

    }

    /**
     * Retorna a imagem atual
     * @return
     */
    public Image getImagem() {
        return imagem;
    }

    /**
     * Retorna a coordenada X para desenhar na posição correta na Tela
     * @return
     */
    public int getX() {
        return x;
    }

    /**
     * Retorna a coordenada Y para desenhar na posição correta na Tela
     * @return
     */
    public int getY() {
        return y;
    }

    /**
     * Definindo uma nova coordenada X para o objeto
     * @param x posicao nova
     */
    public void setX(int x) {
        this.x = x;
    }

    /**
     * Definindo uma nova coordenada y para o objeto
     * @param y posicao nova
     */
    public void setY(int y) {
        this.y = y;
    }

    /**
     * Para incrementar/decrementar a atual coordenada
     * @param ix valor positivo, negativo ou zero
     */
    public void incrementaX(int ix) {
        this.x += ix;
    }

    /**
     * Para incrementar/decrementar a atual coordenada
     * @param ix valor positivo, negativo ou zero
     */
    public void incrementaY(int iy) {
        this.y += iy;
    }

    /**
     * Retorna a largura da imagem
     * @return 
     */
    public int getLargura() {
        return largura;
    }

    /**
     * Retorna a altura da imagem
     * @return
     */
    public int getAltura() {
        return altura;
    }

    /**
     * Usado para detecção de colisões no jogo
     * @return Cria um retangulo com a posicao atual do jogador e com as dimensoes da
     * imagem que fora carregada para esse jogador.
     */
    public Rectangle getLimites() {
        return new Rectangle(x, y, largura, altura);
    }
}
